#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-08-08
import sys
import copy
import json
import datetime
import contextvars
from typing import Callable, Union

from loguru import logger as _logger
from loguru._logger import Logger as _Logger

__all__ = ("logger",)


class StderrDispose:
    """"""

    def __init__(self, handler: _logger, flag: str = "json"):
        """"""
        handler.remove()
        self.flag = flag
        self.__stderr_callback = None
        handler.add(sink=sys.stderr, colorize=True, format=self.__dispose)

    def dispose(self, func: Callable) -> Callable:
        self.__stderr_callback = func
        return func

    def __dispose(self, record: dict) -> str:
        """"""
        if self.__stderr_callback is not None:
            return self.__stderr_callback(record)
        return self.__stderr_dispose(record=record)

    def __stderr_dispose(self, record: dict) -> str:
        """"""
        stderr_formatter = "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | " \
                           "<level>{level: <8}</level> | " \
                           "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - " \
                           "<level>{message}</level>"

        if record["extra"].get(self.flag, False) is True:
            stderr_formatter += " - <level>{extra[serialized]}</level>"

        if "serialized" not in record["extra"]:
            record["extra"]["serialized"] = json.dumps(dict())

        stderr_formatter += "\n"
        return stderr_formatter


class JsonDispose:
    """"""

    def __init__(self, flag: str = "json"):
        """"""
        self.flag = flag
        self.__json_callback = None
        self.handler = _logger.patch(self.__dispose)

    def init(self, sink, level: str = "INFO", encoding: str = "UTF-8", **kwargs) -> None:
        """"""
        if "filter" not in kwargs:
            kwargs["filter"] = lambda x: x["extra"].get(self.flag, False) is True
        self.handler.add(sink=sink, level=level, format="{extra[serialized]}", encoding=encoding, **kwargs)
        return

    def bind(self, **kwargs) -> None:
        """"""
        self.handler = self.handler.bind(**kwargs)
        return

    def dispose(self, func) -> Callable:
        """"""
        self.__json_callback = func
        return func

    def __dispose(self, record: dict) -> None:
        """"""
        if self.__json_callback is not None:
            return self.__json_callback(record=record)
        return self.__json_dispose(record=record)

    def __json_dispose(self, record: dict) -> None:
        """"""
        data = copy.copy(record["extra"])
        data.pop(self.flag, "")
        data.update(**dict(
            message=record.get("message", ""),
            level=record.get("level", dict()).name,
            fileline=":".join([record["name"], record["function"], str(record["line"])]),
            datetime=record.get("time", datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S.%f"),
            timestamp=record.get("time", datetime.datetime.now()).timestamp()
        ))
        record["extra"]["serialized"] = json.dumps(data, ensure_ascii=False)
        return


class AppDispose:
    """"""

    def __init__(self, flag: str = "json"):
        """"""
        self.flag = flag
        self.__app_callback = None

    def init(self, handler: _logger, sink, level: str = "INFO", **kwargs) -> None:
        """"""
        handler.add(sink=sink, level=level, format=self.__dispose, **kwargs)
        return

    def dispose(self, func) -> Callable:
        """"""
        self.__app_callback = func
        return func

    def __dispose(self, record: dict) -> str:
        """"""
        if self.__app_callback is not None:
            return self.__app_callback(record=record)
        return self.__app_dispose(record=record)

    def __app_dispose(self, record: dict) -> str:
        """"""
        stderr_formatter = "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | " \
                           "<level>{level: <8}</level> | " \
                           "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - " \
                           "<level>{message}</level>"

        if record["extra"].get(self.flag, False) is True:
            stderr_formatter += " - <level>{extra[serialized]}</level>"

        if "serialized" not in record["extra"]:
            record["extra"]["serialized"] = json.dumps(dict())

        stderr_formatter += "\n"
        return stderr_formatter


class Logger:
    """"""
    _context_bind = contextvars.ContextVar("context_bind", default="")

    def __init__(self):
        """"""
        self.__flag = "json"
        self.__rename_callback = None
        self.app = AppDispose(flag=self.__flag)
        self.json = JsonDispose(flag=self.__flag)
        self.stderr = StderrDispose(handler=self.json.handler, flag=self.__flag)

    def __getattr__(self, item) -> _logger:
        """"""
        return getattr(self.handler, item)

    @property
    def handler(self) -> _logger:
        context_bind = self._context_bind.get() or None
        if context_bind is not None:
            return self.json.handler.bind(**json.loads(context_bind))
        return self.json.handler

    def init_app(self, sink, level: str = "INFO", **kwargs):
        """"""
        self.app.init(handler=self.json.handler, sink=sink, level=level, **kwargs)
        return

    def init_json(self, sink, level: str = "INFO", encoding: str = "UTF-8", **kwargs) -> None:
        """"""
        self.json.init(sink=sink, level=level, encoding=encoding, **kwargs)
        return

    def global_bind(self, **kwargs) -> None:
        """"""
        self.json.bind(**kwargs)
        return

    def context_bind(self, **kwargs) -> None:
        """"""
        self._context_bind.set(json.dumps(kwargs))
        return

    @property
    def flag(self):
        """"""
        return self.__flag

    def setter_flag(self, flag: str):
        """"""
        self.__flag = flag
        return flag


logger: Union[Logger, _Logger] = Logger()



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-08-11
import json
import os.path
import time

import yaml


class ConfigFile(object):
    """config file module"""

    def __init__(self, config_dir):
        self.config_dir = config_dir

    def is_app_exist(self, sign, suffix):
        """check the app configuration file exists"""
        file_path = os.path.join(self.config_dir, '.'.join([sign, suffix]))
        if not os.path.isfile(file_path):
            return False
        else:
            return file_path

    def load_app(self, sign):
        """load app configuration file"""
        if self.is_app_exist(sign, 'yaml'):
            return self.load_app_yaml(sign)
        elif self.is_app_exist(sign, 'json'):
            return self.load_app_json(sign)
        return None

    @staticmethod
    def load_out(file_path):
        """load outside configuration file"""
        suffix = file_path.split('.')[-1]
        method = 'load_out_' + suffix
        if hasattr(ConfigFile, method):
            return getattr(ConfigFile, method)(file_path)
        else:
            return False

    def load_app_json(self, sign):
        """load_ dynamic configuration file"""
        file_path = self.is_app_exist(sign, 'json')
        return self.load_out_json(file_path)

    @staticmethod
    def load_out_json(file_path):
        """load outside configuration json file"""
        if not os.path.isfile(file_path):
            return False
        with open(file_path) as f:
            return json.loads(f.read())

    def load_dynamic_json(self, sign, callback, expires=3600):
        """load dynamic configuration json file"""
        file_path = self.is_app_exist(sign, 'dynamic.json')
        # file does not exist
        if not file_path:
            file_path = os.path.join(self.config_dir, '.'.join([sign, 'dynamic', 'json']))
            file_content = callback()
            with open(file_path, 'w') as f:
                f.write(file_content)
        # file expired
        st_m_time = os.stat(file_path).st_mtime
        if time.time() > (st_m_time + expires):
            file_content = callback()
            with open(file_path, 'w') as f:
                f.write(file_content)
        # load file
        return self.load_app_json(sign + '.dynamic')

    def del_dynamic_json(self, sign):
        """delete dynamic json file"""
        file_path = os.path.join(self.config_dir, '.'.join([sign, 'dynamic', 'json']))
        if os.path.isfile(file_path):
            os.unlink(file_path)

    def load_app_yaml(self, sign):
        """load app configuration yaml file"""
        file_path = self.is_app_exist(sign, 'yaml')
        return self.load_out_yaml(file_path)

    @staticmethod
    def load_out_yaml(file_path):
        """load outside configuration yaml file"""
        if not os.path.isfile(file_path):
            return False
        with open(file_path, 'rb') as f:
            return yaml.load(f.read(), Loader=yaml.FullLoader)



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-28
import json
import certifi
from urllib.parse import urlencode

from loguru import logger
from urllib3 import PoolManager, HTTPResponse, HTTPConnectionPool

__all__ = ("Requester",)


class Response(HTTPResponse):
    """"""

    def json_decode(self):
        """"""
        return json.loads(self.data)

    def check_http_code(self, value: int):
        """"""
        if self.status != value:
            raise ValueError(f"The target status is {value}, but the actual request status is {self.status}.")
        return self

    def check_json_state(self, field, value):
        """"""
        result = json.loads(self.data)
        if result[field] != value:
            raise ValueError(
                f"The target status is {value}, but the actual request status is {result.get('field', '')}."
            )
        return result


class Requester(PoolManager):
    """"""

    def __init__(self, num_pools=10, headers=None, with_logger=False, **connection_pool_kw):
        """"""

        HTTPConnectionPool.ResponseCls = Response

        if "cert_reqs" not in connection_pool_kw:
            connection_pool_kw["cert_reqs"] = "CERT_REQUIRED"

        if "ca_certs" not in connection_pool_kw:
            connection_pool_kw["ca_certs"] = certifi.where()

        self.with_logger = with_logger
        super(Requester, self).__init__(num_pools=num_pools, headers=headers, **connection_pool_kw)

    def request(self, method, url, fields=None, headers=None, with_logger=False, **urlopen_kw):
        """"""
        with_logger = self.with_logger or with_logger
        if with_logger is True:
            return self._with_logger(method=method, url=url, fields=fields, headers=headers, **urlopen_kw)
        return self._request(method=method, url=url, fields=fields, headers=headers, **urlopen_kw)

    def json_request(self, method, url, fields=None, headers=None, with_logger=False, **urlopen_kw):
        """"""
        if "Content-Type" not in headers:
            headers["Content-Type"] = "application/json; charset=utf-8"

        if "body" not in urlopen_kw:
            raise TypeError(
                "JSON type request, parameters can only be placed in the request body."
            )

        urlopen_kw["body"] = json.dumps(urlopen_kw["body"])

        if fields:
            url += "?" + urlencode(fields)

        with_logger = self.with_logger or with_logger
        if with_logger is True:
            return self._with_logger(method=method, url=url, headers=headers, **urlopen_kw)
        return self._request(method=method, url=url, headers=headers, **urlopen_kw)

    def _request(self, method, url, fields=None, headers=None, **urlopen_kw):
        """
        Make a request using :meth:`urlopen` with the appropriate encoding of
        ``fields`` based on the ``method`` used.

        This is a convenience method that requires the least amount of manual
        effort. It can be used in most situations, while still having the
        option to drop down to more specific methods when necessary, such as
        :meth:`request_encode_url`, :meth:`request_encode_body`,
        or even the lowest level :meth:`urlopen`.
        """
        method = method.upper()

        urlopen_kw["request_url"] = url

        if method in self._encode_url_methods:
            return self.request_encode_url(
                method, url, fields=fields, headers=headers, **urlopen_kw
            )
        else:
            return self.request_encode_body(
                method, url, fields=fields, headers=headers, **urlopen_kw
            )

    def _with_logger(self, method, url, fields=None, headers=None, **urlopen_kw):
        """"""
        fields = fields or dict()
        headers = headers or dict()
        body = urlopen_kw.get("body", dict()) or dict()

        _fields = {
            key: value if isinstance(value, (str, list, int, dict, tuple)) else str(value)
            for key, value in fields.items()
        }

        logger.debug("".join(["*" * 30, " " * 5, "REQUEST START", " " * 5, "*" * 30]))
        logger.debug(f"METHOD: {method}")
        logger.debug(f"URL: {url}")
        logger.debug(f"HEADERS: {json.dumps(headers, ensure_ascii=False)}")
        logger.debug(f"FIELDS: {json.dumps(_fields, ensure_ascii=False)}")
        logger.debug(f"BODY: {body if isinstance(body, str) else json.dumps(body, ensure_ascii=False)}")
        response = self._request(method=method, url=url, fields=fields, headers=headers, **urlopen_kw)
        logger.debug("".join(["*" * 30, " " * 5, "REQUEST END  ", " " * 5, "*" * 30]))
        logger.debug("".join(["*" * 30, " " * 5, "RESPONSE START", " " * 4, "*" * 30]))
        try:
            logger.debug(f"RESPONSE: {json.loads(response.data)}")
        except (UnicodeDecodeError, TypeError, json.JSONDecodeError):
            if isinstance(response.data, bytes):
                _response = response.data if len(response.data) < 30 else \
                    response.data[:15] + b'.....' + response.data[-15:]
            else:
                _response = response.data if len(response.data) < 30 else \
                    response.data[:15] + '......' + response.data[-15:]
            logger.debug(f"RESPONSE: {_response}")
        except Exception as e:
            logger.warning(
                f"JSON deserialization found location error, unable to print the request result log, "
                f"but does not affect the program operation. - {e}"
            )
            logger.debug("RESPONSE: Unknown error occurred in JSON parsing, please use the response result normally.")
        logger.debug("".join(["*" * 30, " " * 5, "RESPONSE END ", " " * 5, "*" * 30]))
        return response



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-28
from fourcats_utils.logger import logger
from fourcats_utils.registry import Registry
from fourcats_utils.requester import Requester
from fourcats_utils.configfile import ConfigFile

__version__ = "0.0.1"
__all__ = ("logger", "Registry", "Requester", "ConfigFile")



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-08-11
from mergedict import MergeDict


class Registry(object):
    """ Configuration module"""

    def __init__(self, options=None):
        if type(options) is dict:
            self.options = options
        else:
            self.options = {}
        self.hooks = {}

    def set(self, key, value):
        """Set configuration items."""
        items = key.split('.')
        if len(items) == 1:
            self.options[key] = value
            return self.get(key)
        end_key = items.pop()
        options = self.options
        for item in items:
            if options.get(item) is None:
                options.setdefault(item, {})
            if not isinstance(options.get(item), dict):
                options[item] = {}
            options = options.get(item)
        options[end_key] = value
        return self.get(key)

    def merge(self, key, value):
        """Merge configuration items."""
        if not isinstance(value, dict):
            return False
        news = MergeDict(self.get(key, {}))
        news.merge(value)
        self.set(key, news)
        return news

    def get(self, key=None, default=None, empty=False):
        """Get configuration items."""
        if key is None:
            return self.options
        items = key.split('.')
        options = self.options
        for item in items:
            options = options.get(item)
            if options is None:
                return default
        if empty:
            if not options:
                return default
        return options

    def default(self, key=None, default=None):
        """Set default values."""
        value = self.get(key)
        if value is None:
            self.get(key, default)
            return default
        return value

    def load(self, value):
        """Load configuration items."""
        if isinstance(value, dict):
            self.options.update(value)
            return True
        else:
            return False



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-08-11
# !/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-23
import re
from setuptools import find_packages, setup

with open("fourcats_utils/__init__.py", "r") as file:
    regex_version = r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]'
    version = re.search(regex_version, file.read(), re.MULTILINE).group(1)

with open("README.md", "rb") as file:
    readme = file.read().decode("utf-8")

setup(
    name="fourcats-utils",
    version=version,
    packages=find_packages(),
    description="A common Python toolkit package based on personal habits.",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="ShiWeiDong",
    author_email="shiweidong1993@gmail.com",
    url="https://github.com/FourCats-Py/FourCats-Utils",
    download_url="https://github.com/FourCats-Py/FourCats-Utils/archive/{}.tar.gz".format(version),
    keywords=["fourcats", "utils"],
    install_requires=["loguru>=0.6.0", "mergedict>=1.0.0", "PyYAML>=6.0", "urllib3>=1.26.11"],
    python_requires=">=3.8"
)
